@@ -8,7 +8,6 @@ from pythonwhat.parsing import (
 from protowhat.State import State as ProtoState
 from protowhat.selectors import DispatcherInterface
 from protowhat.Feedback import InstructorError
-from pythonwhat.Feedback import Feedback
 from pythonwhat import signatures
 from pythonwhat.converters import get_manual_converters
 from collections.abc import Mapping
@@ -63,7 +62,7 @@ class State(ProtoState):
         highlight=None,
         highlighting_disabled=None,
         messages=None,
-        parent_state=None,
+        creator=None,
         student_ast=None,
         solution_ast=None,
         student_ast_tokens=None,
@@ -79,10 +78,12 @@ class State(ProtoState):
         self.params = list()
 
         for k, v in args.items():
-            if k not in ["self", "args"]:
+            if k != "self":
                 self.params.append(k)
                 setattr(self, k, v)
 
+        self.messages = messages if messages else []
+
         self.ast_dispatcher = self.get_dispatcher()
 
         # parse code if didn't happen yet
@@ -91,11 +92,9 @@ class State(ProtoState):
         if solution_ast is None:
             self.solution_ast = self.parse(solution_code, test=False)
 
-        if highlight is None and parent_state:
+        if highlight is None:  # todo: check parent_state? (move check to reporting?)
             self.highlight = self.student_ast
 
-        self.messages = messages if messages else []
-
         self.converters = get_manual_converters()  # accessed only from root state
 
         self.manual_sigs = None
@@ -123,7 +122,6 @@ class State(ProtoState):
             append_message = {"msg": append_message, "kwargs": {}}
 
         kwargs["messages"] = [*self.messages, append_message]
-        kwargs["parent_state"] = self
 
         for kwarg in ["solution_context", "student_context"]:
             if kwarg in kwargs and not kwargs[kwarg]:
@@ -204,26 +202,21 @@ class State(ProtoState):
             e.filename = "script.py"
             # no line info for now
             self.report(
-                Feedback(
-                    "Your code could not be parsed due to an error in the indentation:<br>`%s.`"
-                    % str(e)
-                )
+                "Your code could not be parsed due to an error in the indentation:<br>`%s.`"
+                % str(e)
             )
 
         except SyntaxError as e:
             e.filename = "script.py"
             # no line info for now
             self.report(
-                Feedback(
-                    "Your code can not be executed due to a syntax error:<br>`%s.`"
-                    % str(e)
-                )
+                "Your code can not be executed due to a syntax error:<br>`%s.`" % str(e)
             )
 
         # Can happen, can't catch this earlier because we can't differentiate between
         # TypeError in parsing or TypeError within code (at runtime).
         except:
-            self.report(Feedback("Something went wrong while parsing your code."))
+            self.report("Something went wrong while parsing your code.")
 
         return res
 
@@ -264,14 +257,12 @@ class Dispatcher(DispatcherInterface):
         self._parser_cache = dict()
         context_ast = getattr(self._context_cache, context_code, None)
         if context_ast is None:
-            context_ast = self._context_cache[context_code] = self.parse(
-                context_code
-            )[1]
-        self.context_mappings = self._getx(
-            FunctionParser, "mappings", context_ast
-        )
+            context_ast = self._context_cache[context_code] = self.parse(context_code)[
+                1
+            ]
+        self.context_mappings = self._getx(FunctionParser, "mappings", context_ast)
 
-    def __call__(self, name, node, *args, **kwargs):
+    def find(self, name, node, *args, **kwargs):
         return getattr(self, name)(node)
 
     def parse(self, code):
