@@ -16,7 +16,10 @@ as well as some extra documentation:
     https://greentreesnakes.readthedocs.org/en/latest/
 """
 
-class EmptyTargetVar: pass
+
+class EmptyTargetVar:
+    pass
+
 
 class TargetVars(Mapping):
     """Immutable ordered mapping from target variables to their values."""
@@ -24,15 +27,20 @@ class TargetVars(Mapping):
     EMPTY = EmptyTargetVar()
 
     def __init__(self, target_vars=tuple(), is_empty=True):
-        if is_empty: 
+        if is_empty:
             target_vars = [(v, self.EMPTY) for v in target_vars]
 
         self._od = OrderedDict(target_vars)
 
     # getitem, len, iter wrap OrderedDict behavior
-    def __getitem__(self, k): return self._od.__getitem__(k)
-    def __len__(self):        return self._od.__len__()
-    def __iter__(self):       return self._od.__iter__()
+    def __getitem__(self, k):
+        return self._od.__getitem__(k)
+
+    def __len__(self):
+        return self._od.__len__()
+
+    def __iter__(self):
+        return self._od.__iter__()
 
     def update(self, *args, **kwargs):
         cpy = self.copy()
@@ -44,12 +52,17 @@ class TargetVars(Mapping):
 
     def __str__(self):
         """Format target vars for printing"""
-        if len(self) >  1: return "({})".format(", ".join(self._od.keys()))
-        else: return "".join(self._od.keys())
+        if len(self) > 1:
+            return "({})".format(", ".join(self._od.keys()))
+        else:
+            return "".join(self._od.keys())
 
     def defined_items(self):
         """Return copy of instance, omitting entries that are EMPTY"""
-        return self.__class__([(k, v) for k,v in self.items() if v is not self.EMPTY], is_empty=False)
+        return self.__class__(
+            [(k, v) for k, v in self.items() if v is not self.EMPTY], is_empty=False
+        )
+
 
 class IndexedDict(Mapping):
     """Wrapper around OrderedDict that allows access via item position or key"""
@@ -57,15 +70,19 @@ class IndexedDict(Mapping):
     def __init__(self, *args, **kwargs):
         self._od = OrderedDict(*args, **kwargs)
 
-    def __getitem__(self, k): 
-        try: return list(self._od.values())[k]
-        except TypeError: return self._od[k]
-    
-    def __len__(self):  return self._od.__len__()
-    def __iter__(self): return self._od.__iter__()
+    def __getitem__(self, k):
+        try:
+            return list(self._od.values())[k]
+        except TypeError:
+            return self._od[k]
+
+    def __len__(self):
+        return self._od.__len__()
+
+    def __iter__(self):
+        return self._od.__iter__()
 
 
-        
 class Parser(ast.NodeVisitor):
     """Basic parser.
 
@@ -106,7 +123,7 @@ class Parser(ast.NodeVisitor):
             node (ast.Node): The node which is visited.
         """
         pass  # This ignore is necessary to keep the parser at base level, also look comment above in
-              # the visit_Module function body.
+        # the visit_Module function body.
 
     def visit_each(self, lst):
         for el in lst:
@@ -115,11 +132,12 @@ class Parser(ast.NodeVisitor):
     @staticmethod
     def get_target_vars(target):
         get_id = lambda n: n.id if not isinstance(n, ast.Starred) else n.value.id
-        if isinstance(target, (ast.Name, ast.Starred)):    
+        if isinstance(target, (ast.Name, ast.Starred)):
             tv = [get_id(target)]
-        elif isinstance(target, ast.Tuple): 
+        elif isinstance(target, ast.Tuple):
             tv = [get_id(node) for node in target.elts]
-        else: tv = []
+        else:
+            tv = []
 
         return TargetVars(tv)
 
@@ -127,7 +145,7 @@ class Parser(ast.NodeVisitor):
     def get_arg(el):
         if el is None:
             return None
-        else :
+        else:
             return el.arg
 
     @staticmethod
@@ -138,7 +156,7 @@ class Parser(ast.NodeVisitor):
 
     @staticmethod
     def get_arg_parts(arguments, defaults, type):
-        # only difference is that it doesn't pull out arg.arg, so we can 
+        # only difference is that it doesn't pull out arg.arg, so we can
         # use all the information on the arg node down the road
         match_def = [None] * (len(arguments) - len(defaults)) + defaults
         part_list = []
@@ -149,18 +167,19 @@ class Parser(ast.NodeVisitor):
     @staticmethod
     def get_arg_part(_arg, _def, type=None):
         # type is arg, kwonly, kwarg, vararg
-        if not _arg: return None
+        if not _arg:
+            return None
 
         # part uses default highlighting, so will highlight "node" entry
         return {
-                'node': _def or _arg,
-                'arg': _arg,
-                # TODO: need to fill out
-                'type': type,
-                'is_default': True if _def else False,
-                'name': _arg.arg,
-                'annotation': _arg.annotation
-                }
+            "node": _def or _arg,
+            "arg": _arg,
+            # TODO: need to fill out
+            "type": type,
+            "is_default": True if _def else False,
+            "name": _arg.arg,
+            "annotation": _arg.annotation,
+        }
 
 
 # class OperatorParser(Parser):
@@ -261,8 +280,8 @@ class FunctionParser(Parser):
     """
 
     def __init__(self):
-        self.gen_name = ''
-        self.raw_name = ''
+        self.gen_name = ""
+        self.raw_name = ""
         self.mappings = {}
         self.out = {}
         self.call_lookup_active = False
@@ -288,7 +307,7 @@ class FunctionParser(Parser):
             if imp.asname is not None:
                 self.mappings[imp.asname] = imp.name
             else:
-                pass # e.g. numpy import as numpy, so no action needed.
+                pass  # e.g. numpy import as numpy, so no action needed.
 
     def visit_ImportFrom(self, node):
         for imp in node.names:
@@ -298,24 +317,26 @@ class FunctionParser(Parser):
         self.visit(node.value)
 
     def visit_List(self, node):
-        [ self.visit(el) for el in node.elts ]
+        [self.visit(el) for el in node.elts]
 
     def visit_Dict(self, node):
-        [ self.visit(el) for el in node.values ]
+        [self.visit(el) for el in node.values]
 
     def visit_Call(self, node):
         if self.call_lookup_active:
             self.visit(node.func)
-        else :
+        else:
             self.call_lookup_active = True
-            self.visit(node.func) # Need to visit func to start recording the current function name.
+            self.visit(
+                node.func
+            )  # Need to visit func to start recording the current function name.
 
             if self.gen_name:
-                if (self.gen_name not in self.out):
+                if self.gen_name not in self.out:
                     self.out[self.gen_name] = []
 
                 self.out[self.gen_name].append(self.get_call_part(node))
-                #self.out[self.current].append((node, node.args, node.keywords))
+                # self.out[self.current].append((node, node.args, node.keywords))
 
             self.gen_name = self.raw_name = ""
             self.call_lookup_active = False
@@ -341,44 +362,46 @@ class FunctionParser(Parser):
     def visit_Name(self, node):
         self.gen_name = self.mappings.get(node.id) or node.id
         self.raw_name = node.id
-    
+
     def get_call_part(self, node):
         args = [self.get_pos_arg_part(n, ii) for ii, n in enumerate(node.args)]
         keywords = [self.get_kw_arg_part(n) for n in node.keywords]
-        return {'node': node,
-                # TODO: right now, args and keywords can be indexed by pos or name.
-                #       Note that a pos args name is its position.
-                #       Problems will arise if SCT tests a position, but the submission
-                #       has too few positional arguments, since it will then grab a kw arg :(
-                #       This is not necessarily a bad thing, but instructors would need to be
-                #       Careful deciding when to test a pos arg, and when to test using kw.
-                #       Could use check_pos_args with pos_args entry below to solve.
-                'args': IndexedDict((n['name'], n) for n in [*args, *keywords]),
-                #'pos_args': args,
-                #'keywords': keywords,
-                'name': self.raw_name
-                }
+        return {
+            "node": node,
+            # TODO: right now, args and keywords can be indexed by pos or name.
+            #       Note that a pos args name is its position.
+            #       Problems will arise if SCT tests a position, but the submission
+            #       has too few positional arguments, since it will then grab a kw arg :(
+            #       This is not necessarily a bad thing, but instructors would need to be
+            #       Careful deciding when to test a pos arg, and when to test using kw.
+            #       Could use check_pos_args with pos_args entry below to solve.
+            "args": IndexedDict((n["name"], n) for n in [*args, *keywords]),
+            #'pos_args': args,
+            #'keywords': keywords,
+            "name": self.raw_name,
+        }
 
-    def get_pos_arg_part(self, arg, indx_pos):
+    @staticmethod
+    def get_pos_arg_part(arg, indx_pos):
         is_star = isinstance(arg, ast.Starred)
         return {
-                'node': arg if not is_star else arg.value,
-                'highlight': arg,
-                'type': 'argument',
-                'is_starred': is_star,
-                'name': indx_pos
-                }
+            "node": arg if not is_star else arg.value,
+            "highlight": arg,
+            "type": "argument",
+            "is_starred": is_star,
+            "name": indx_pos,
+        }
 
-    def get_kw_arg_part(self, arg):
+    @staticmethod
+    def get_kw_arg_part(arg):
         is_kwarg = arg.arg is None
         return {
-                'node': arg.value,
-                'highlight': arg,
-                'type': 'keyword',
-                'is_kwarg': is_kwarg,
-                'name': arg.arg
-                }
-
+            "node": arg.value,
+            "highlight": arg,
+            "type": "keyword",
+            "is_kwarg": is_kwarg,
+            "name": arg.arg,
+        }
 
 
 class ObjectAccessParser(FunctionParser):
@@ -416,12 +439,13 @@ class ObjectAccessParser(FunctionParser):
         # if name refers to an import, replace
         prefix = self.mappings.get(node.id) or node.id
 
-        self.gen_name = prefix  + "." + self.gen_name if self.gen_name else prefix
+        self.gen_name = prefix + "." + self.gen_name if self.gen_name else prefix
         self.raw_name = node.id + "." + self.raw_name if self.raw_name else node.id
 
         self.out.append(self.gen_name)
         self.gen_name = self.raw_name = ""
 
+
 class ObjectAssignmentParser(Parser):
     """Find object assignmnts
 
@@ -438,7 +462,7 @@ class ObjectAssignmentParser(Parser):
             if node.id not in self.out:
                 self.out[node.id] = self.get_part(node, self.active_assignment)
             else:
-                self.out[node.id]['highlight'] = None
+                self.out[node.id]["highlight"] = None
             self.active_assignment = None
 
     def visit_Attribute(self, node):
@@ -474,14 +498,10 @@ class ObjectAssignmentParser(Parser):
     @staticmethod
     def get_part(name_node, ass_node=None):
         # either name node or simply str or name itself
-        name = getattr(name_node, 'id', name_node)
+        name = getattr(name_node, "id", name_node)
         load_name = ast.Name(id=name, ctx=ast.Load())
         ast.fix_missing_locations(load_name)
-        return {
-            'name': name,
-            'node': load_name,
-            'highlight': ass_node or name_node,
-        }
+        return {"name": name, "node": load_name, "highlight": ass_node or name_node}
 
 
 class IfParser(Parser):
@@ -495,12 +515,9 @@ class IfParser(Parser):
         self.out = []
 
     def visit_If(self, node):
-        self.out.append({
-            'node': node,
-            'test': node.test,
-            'body': node.body,
-            'orelse': node.orelse,
-            })
+        self.out.append(
+            {"node": node, "test": node.test, "body": node.body, "orelse": node.orelse}
+        )
 
 
 class IfExpParser(IfParser):
@@ -510,7 +527,8 @@ class IfExpParser(IfParser):
     Only 'top-level' if structures will be found!
     """
 
-    def visit_If(self, node): return
+    def visit_If(self, node):
+        return
 
     def visit_IfExp(self, node):
         super().visit_If(node)
@@ -543,12 +561,9 @@ class WhileParser(Parser):
         self.out = []
 
     def visit_While(self, node):
-        self.out.append({
-            'node': node,
-            'test': node.test,
-            'body': node.body,
-            'orelse': node.orelse
-            })
+        self.out.append(
+            {"node": node, "test": node.test, "body": node.body, "orelse": node.orelse}
+        )
 
 
 class ForParser(Parser):
@@ -563,14 +578,17 @@ class ForParser(Parser):
 
     def visit_For(self, node):
         tv = Parser.get_target_vars(node.target)
-        self.out.append({
-            'node': node,
-            'iter': node.iter,
-            'body': {'node': node.body, 'target_vars': tv},
-            'orelse': {'node': node.orelse, 'target_vars': tv},
-            'target': node.target,
-            '_target_vars': tv
-            })
+        self.out.append(
+            {
+                "node": node,
+                "iter": node.iter,
+                "body": {"node": node.body, "target_vars": tv},
+                "orelse": {"node": node.orelse, "target_vars": tv},
+                "target": node.target,
+                "_target_vars": tv,
+            }
+        )
+
 
 class ClassDefParser(Parser):
     """Find class definitions
@@ -581,17 +599,19 @@ class ClassDefParser(Parser):
 
     def visit_ClassDef(self, node):
         self.out[node.name] = {
-            'node': node,
-            'bases': [ {'node': node } for node in node.bases ],
-            'body': node.body,
+            "node": node,
+            "bases": [{"node": node} for node in node.bases],
+            "body": node.body,
         }
 
+
 class FunctionDefParser(Parser):
     """Find function definitions
 
     A parser which inherits from the basic parser to find function definitions.
     Only 'top-level' for structures will be found!
     """
+
     def __init__(self):
         self.out = {}
 
@@ -604,33 +624,37 @@ class FunctionDefParser(Parser):
         kwonlyargs = cls.get_arg_tuples(node.args.kwonlyargs, node.args.kw_defaults)
         # TODO: all single args should be tuples like this
         vararg = cls.get_arg(node.args.vararg)
-        kwarg =  cls.get_arg(node.args.kwarg)
+        kwarg = cls.get_arg(node.args.kwarg)
         # create context variables
         target_vars = [arg[0] for arg in normal_args]
-        if vararg: target_vars.append(vararg)
-        if kwarg:  target_vars.append(kwarg)
+        if vararg:
+            target_vars.append(vararg)
+        if kwarg:
+            target_vars.append(kwarg)
 
-        args = cls.get_arg_parts(node.args.args, node.args.defaults, 'arg')
-        kw_args = cls.get_arg_parts(node.args.kwonlyargs, node.args.kw_defaults, 'kwonly')
-        varargs = cls.get_arg_part(node.args.vararg, None, 'vararg')
-        kwargs = cls.get_arg_part(node.args.kwarg, None, 'kwarg')
+        args = cls.get_arg_parts(node.args.args, node.args.defaults, "arg")
+        kw_args = cls.get_arg_parts(
+            node.args.kwonlyargs, node.args.kw_defaults, "kwonly"
+        )
+        varargs = cls.get_arg_part(node.args.vararg, None, "vararg")
+        kwargs = cls.get_arg_part(node.args.kwarg, None, "kwarg")
         all_args = [*args, varargs, *kw_args, kwargs]
-        
-        if isinstance(node, ast.Lambda): body_node = node.body
+
+        if isinstance(node, ast.Lambda):
+            body_node = node.body
         else:
             bodyMod = wrap_in_module(node.body)
             body_node = FunctionBodyTransformer().visit(bodyMod)
 
         return {
             "node": node,
-            "name": getattr(node, 'name', None),
-            "args": IndexedDict([ (p['name'], p) for p in all_args if p is not None]),
+            "name": getattr(node, "name", None),
+            "args": IndexedDict([(p["name"], p) for p in all_args if p is not None]),
             # TODO: arg is the node counterpart to target_vars
             "_spec1_args": args,
             "*args": varargs,
-            "**kwargs":  kwargs,
-            "body": {'node': body_node, 
-                     'target_vars': TargetVars(target_vars)}
+            "**kwargs": kwargs,
+            "body": {"node": body_node, "target_vars": TargetVars(target_vars)},
         }
 
 
@@ -674,22 +698,26 @@ class CompParser(Parser):
         target = node.generators[0].target
         tv = Parser.get_target_vars(target)
         ifs = node.generators[0].ifs
-        self.out.append({
+        self.out.append(
+            {
                 "node": node,
-                "body": {'node': node.elt, 'target_vars': tv},
+                "body": {"node": node.elt, "target_vars": tv},
                 "target": target,
                 "iter": node.generators[0].iter,
-                "ifs":  [{'node': ifnode, 'target_vars': tv} for ifnode in ifs],
+                "ifs": [{"node": ifnode, "target_vars": tv} for ifnode in ifs],
                 # TODO: 'private' _target_vars, since it shouldn't be set when selecting node,
                 #       see remarks in test_list_comp on rewriting
-                "_target_vars": tv
-            })
+                "_target_vars": tv,
+            }
+        )
+
 
 class ListCompParser(CompParser):
     """Find list comprehensions
 
     A parser which inherits from the CompParser to find list comprehensions.
     """
+
     def visit_ListComp(self, node):
         self.build_comp(node)
 
@@ -703,36 +731,40 @@ class GeneratorExpParser(CompParser):
     def visit_GeneratorExp(self, node):
         self.build_comp(node)
 
+
 class DictCompParser(CompParser):
     """Find dictionary comprehensions
 
     A parser which inherits from the CompParser to find dict comprehensions.
     """
+
     def visit_DictComp(self, node):
         target = node.generators[0].target
         tv = Parser.get_target_vars(target)
         ifs = node.generators[0].ifs
-        self.out.append({
+        self.out.append(
+            {
                 "node": node,
-                "key": {'node': node.key, 'target_vars': tv},
-                "value": {'node': node.value, 'target_vars': tv},
+                "key": {"node": node.key, "target_vars": tv},
+                "value": {"node": node.value, "target_vars": tv},
                 "target": target,
                 "iter": node.generators[0].iter,
-                "ifs": [{'node': ifnode, 'target_vars': tv} for ifnode in ifs],
+                "ifs": [{"node": ifnode, "target_vars": tv} for ifnode in ifs],
                 # TODO: 'private' _target_vars, since it shouldn't be set when selecting node,
                 #       see remarks in test_list_comp on rewriting
-                "_target_vars": tv
-            })
+                "_target_vars": tv,
+            }
+        )
 
 
 class FunctionBodyTransformer(ast.NodeTransformer):
     # TODO this does not automatically contain line_end information!
     def visit_Nonlocal(self, node):
-        new_node = ast.copy_location(ast.Global(names = node.names), node)
+        new_node = ast.copy_location(ast.Global(names=node.names), node)
         return FunctionBodyTransformer.decorate(new_node, node)
 
     def visit_Return(self, node):
-        new_node = ast.copy_location(ast.Expr(value = node.value), node)
+        new_node = ast.copy_location(ast.Expr(value=node.value), node)
         return FunctionBodyTransformer.decorate(new_node, node)
 
     @staticmethod
@@ -741,6 +773,7 @@ class FunctionBodyTransformer(ast.NodeTransformer):
         new_node.last_token = node.last_token
         return new_node
 
+
 class WithParser(Parser):
     def __init__(self):
         self.out = []
@@ -748,21 +781,26 @@ class WithParser(Parser):
     def visit_With(self, node):
         items = node.items
         context = [
-            {"node" : item.context_expr,
-             "target_vars": self.get_target_vars(item.optional_vars),
-             "with_items": item,
-             "highlight": node
-                } 
-            for item in items]
+            {
+                "node": item.context_expr,
+                "target_vars": self.get_target_vars(item.optional_vars),
+                "with_items": item,
+                "highlight": node,
+            }
+            for item in items
+        ]
 
-        tv_all = TargetVars(sum([list(c['target_vars'].items()) for c in context], []))
+        tv_all = TargetVars(sum([list(c["target_vars"].items()) for c in context], []))
+
+        self.out.append(
+            {
+                "context": context,
+                "body": {"node": node.body, "with_items": items},
+                "node": node,
+                "n_vars": len(items),
+            }
+        )
 
-        self.out.append({
-            "context": context,
-            "body": {'node': node.body, 'with_items': items},
-            "node": node,
-            "n_vars": len(items)
-        })
 
 class TryExceptParser(Parser):
     def __init__(self):
@@ -778,22 +816,23 @@ class TryExceptParser(Parser):
                     handlers[el.id] = self.parse_handler(handler)
             else:
                 # either general handler, or single error handler
-                k = 'all' if not handler.type else handler.type.id
+                k = "all" if not handler.type else handler.type.id
                 handlers[k] = self.parse_handler(handler)
 
-        self.out.append({
-            "node": node,
-            "body": node.body,
-            "orelse": node.orelse or None,
-            "finalbody": node.finalbody or None,
-            "handlers": handlers,
-        })
+        self.out.append(
+            {
+                "node": node,
+                "body": node.body,
+                "orelse": node.orelse or None,
+                "finalbody": node.finalbody or None,
+                "handlers": handlers,
+            }
+        )
 
     @staticmethod
-    def parse_handler(handler): return {
-                'node': handler.body,
-                'target_vars': TargetVars([handler.name])
-                }
+    def parse_handler(handler):
+        return {"node": handler.body, "target_vars": TargetVars([handler.name])}
+
 
 parser_dict = {
     "object_accesses": ObjectAccessParser,
@@ -812,5 +851,5 @@ parser_dict = {
     "generator_exps": GeneratorExpParser,
     "withs": WithParser,
     "try_excepts": TryExceptParser,
-    "function_calls": FunctionParser
+    "function_calls": FunctionParser,
 }
