@@ -1,3 +1,4 @@
+from protowhat.sct_syntax import link_to_state
 from pythonwhat.checks.check_funcs import (
     check_node,
     check_part,
@@ -16,6 +17,12 @@ from pythonwhat.checks.check_has_context import has_context
 from functools import partial
 from pythonwhat import utils
 
+# this is done by the chain for v2
+# it's only needed when a new state is created and (possibly) used elsewhere
+check_node = link_to_state(check_node)
+check_part = link_to_state(check_part)
+check_part_index = link_to_state(check_part_index)
+
 
 def test_if_else(state, index=1, test=None, body=None, orelse=None):
     """Test parts of the if statement.
@@ -275,19 +282,17 @@ def test_function_definition(
             test_function_definition('shout')                          # fail
             test_function_definition('shout', arg_defaults = False)    # pass
             test_function_definition('shout', arg_defaults = False,    # fail
-                                            outputs = [('help')])    
+                                            outputs = [('help')])
 
             test_function_definition('shout', arg_defaults = False,    # pass
-                                            results = [('help', 2)]) 
+                                            results = [('help', 2)])
 
             test_function_definition('shout', args_defaults = False    # pass
                     body = test_function('print', args = []]))
     """
 
     # what the function will be referred to as
-    child = check_node(
-        state, "function_defs", name, "definition of `{{index}}()`"
-    )
+    child = check_node(state, "function_defs", name, "definition of `{{index}}()`")
 
     test_args(
         child, arg_names, arg_defaults, nb_args_msg, arg_names_msg, arg_defaults_msg
@@ -332,7 +337,7 @@ def test_function_definition(
 
 
 def test_args(
-        state, arg_names, arg_defaults, nb_args_msg, arg_names_msg, arg_defaults_msg
+    state, arg_names, arg_defaults, nb_args_msg, arg_names_msg, arg_defaults_msg
 ):
 
     MSG_NUM_ARGS = "You should define {{parent[typestr]}} with {{sol_len}} arguments, instead got {{stu_len}}."
@@ -439,7 +444,11 @@ def test_object_after_expression(
     pre_code=None,
     **kwargs
 ):
-    state.highlight = state.ast_dispatcher("object_assignments", state.student_ast).get(name, {}).get("highlight")
+    state.highlight = (
+        state.ast_dispatcher.find("object_assignments", state.student_ast)
+        .get(name, {})
+        .get("highlight")
+    )
     has_equal_value(
         state,
         incorrect_msg=incorrect_msg,
@@ -489,9 +498,7 @@ with open_file('...') as file:
     if context_vals:
         # test context var names ----
         has_context(
-            child,
-            incorrect_msg=context_vals_msg or MSG_CTXT_NAMES,
-            exact_names=True,
+            child, incorrect_msg=context_vals_msg or MSG_CTXT_NAMES, exact_names=True
         )
 
         # test num context vars ----
@@ -504,7 +511,8 @@ with open_file('...') as file:
     for i, context_test in enumerate(context_tests or []):
         # partial the substate check, because the function uses two prepended messages
         def check_context(state):
-            return check_part_index(state,
+            return check_part_index(
+                state,
                 "context",
                 i,
                 "%s context" % utils.get_ord(i + 1),
@@ -536,8 +544,8 @@ def test_list_comp(
 ):
     """Test list comprehension."""
     kwargs = locals().copy()
-    kwargs['typestr'] = "{{ordinal}} list comprehension"
-    kwargs['comptype'] = "list_comps"
+    kwargs["typestr"] = "{{ordinal}} list comprehension"
+    kwargs["comptype"] = "list_comps"
     test_comp(**kwargs)
 
 
@@ -567,9 +575,7 @@ def test_comp(
         insufficient_ifs_msg = MSG_INSUFFICIENT_IFS
 
     # get comprehension
-    child = check_node(
-        state, comptype, index - 1, typestr, missing_msg=not_called_msg
-    )
+    child = check_node(state, comptype, index - 1, typestr, missing_msg=not_called_msg)
 
     # test comprehension iter and its variable names (or number of variables)
     if comp_iter:
@@ -594,7 +600,4 @@ def test_comp(
         # test that ifs are same length
         has_equal_part_len(child, "ifs", insufficient_ifs_msg)
         # test individual ifs
-        multi(
-            check_part_index(child, "ifs", i, utils.get_ord(i + 1) + " if"),
-            if_test,
-        )
+        multi(check_part_index(child, "ifs", i, utils.get_ord(i + 1) + " if"), if_test)
